@@ -7,6 +7,7 @@
 #include "ncd2dispatch.h"
 #include "ncrc.h"
 #include "ncoffsets.h"
+#include "netcdf_dispatch.h"
 #ifdef DEBUG2
 #include "dapdump.h"
 #endif
@@ -178,6 +179,10 @@ NCD2_get_var_chunk_cache,
 NC_NOOP_inq_var_filter_ids,
 NC_NOOP_inq_var_filter_info,
 
+NC_NOTNC4_def_var_quantize,
+NC_NOTNC4_inq_var_quantize,
+
+NC_NOOP_inq_filter_avail,
 };
 
 const NC_Dispatch* NCD2_dispatch_table = NULL; /* moved here from ddispatch.c */
@@ -403,9 +408,8 @@ fprintf(stderr,"ce=%s\n",dumpconstraint(dapcomm->oc.dapconstraint));
     ocstat = oc_open(dapcomm->oc.urltext,&dapcomm->oc.conn);
     if(ocstat != OC_NOERR) {THROWCHK(ocstat); goto done;}
 
-#ifdef DEBUG1
-    (void)oc_trace_curl(dapcomm->oc.conn);
-#endif
+    if(getenv("CURLOPT_VERBOSE") != NULL)
+        (void)oc_trace_curl(dapcomm->oc.conn);
 
     nullfree(dapcomm->oc.urltext); /* clean up */
     dapcomm->oc.urltext = NULL;
@@ -1306,8 +1310,8 @@ applyclientparams(NCDAPCOMMON* nccomm)
 	strlcat(tmpname,pathstr,sizeof(tmpname));
 	value = paramlookup(nccomm,tmpname);
 	if(value == NULL) {
-	    strcpy(tmpname,"maxstrlen_");
-	    strncat(tmpname,pathstr,NC_MAX_NAME);
+	    strncpy(tmpname,"maxstrlen_",sizeof(tmpname));
+	    strlcat(tmpname,pathstr,sizeof(tmpname));
 	    value = paramlookup(nccomm,tmpname);
         }
 	nullfree(pathstr);
@@ -1626,7 +1630,6 @@ getseqdimsize(NCDAPCOMMON* dapcomm, CDFnode* seq, size_t* sizep)
     NCerror ncstat = NC_NOERR;
     OCerror ocstat = OC_NOERR;
     OClink conn = dapcomm->oc.conn;
-    OCdatanode rootcontent = NULL;
     OCddsnode ocroot;
     CDFnode* dxdroot;
     CDFnode* xseq;
@@ -1685,7 +1688,6 @@ fprintf(stderr,"sequencesize: %s = %lu\n",seq->ocname,(unsigned long)seqsize);
 
 fail:
     ncbytesfree(seqcountconstraints);
-    oc_data_free(conn,rootcontent);
     if(ocstat != OC_NOERR) ncstat = ocerrtoncerr(ocstat);
     return ncstat;
 }
