@@ -1,10 +1,10 @@
 /*********************************************************************
- *   Copyright 1993, UCAR/Unidata
+ *   Copyright 2018, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
  *   $Header: /upc/share/CVS/netcdf-3/ncdump/vardata.c,v 1.48 2010/05/05 22:15:39 dmh Exp $
  *********************************************************************/
 
-#include <config.h>
+#include "config.h"
 #include <stdio.h>
 #include <ctype.h>
 #include <stdlib.h>
@@ -12,17 +12,22 @@
 #include <assert.h>
 #include <netcdf.h>
 #include "utils.h"
+#include "netcdf.h"
 #include "nccomps.h"
 #include "dumplib.h"
 #include "ncdump.h"
 #include "indent.h"
 #include "vardata.h"
+#include "netcdf_aux.h"
 
 /* maximum len of string needed for one value of a primitive type */
 #define MAX_OUTPUT_LEN 100
 
 #define LINEPIND	"    "	/* indent of continued lines */
 
+#define LBRACE "{"
+#define RBRACE "}"
+
 extern fspec_t formatting_specs; /* set from command-line options */
 
 /* Only read this many values at a time, if last dimension is larger
@@ -46,11 +51,11 @@ set_max_len(int len) {
 }
 
 
-/* 
+/*
  * Output a string that should not be split across lines.  If it would
  * make current line too long, first output a newline and current
  * (nested group) indentation, then continuation indentation, then
- * output string.  If string ends with a newline to force short line, 
+ * output string.  If string ends with a newline to force short line,
  * reset indentation after output.
  */
 void
@@ -64,7 +69,7 @@ lput(const char *cp) {
 	linep = (int)strlen(LINEPIND) + indent_get();
     }
     (void) fputs(cp,stdout);
-    if (cp[nn - 1] == '\n') {
+    if (nn > 0 && cp[nn - 1] == '\n') {
 	linep = indent_get();
     } else
 	linep += nn;
@@ -98,19 +103,19 @@ lput2(
     					/*   chars); saved between calls    */
     int len_prefix = strlen (CDL_COMMENT_PREFIX);
     bool_t make_newline;
-    
+
     size_t len1 = strlen(cp);		/* length of input string */
 
     assert (len1 > 0);
 
 /* (1) Single space or newline/indent sequence, as needed. */
-    
+
     linep = linep + 1 + len1;		/* new line position, without newline */
-    					/* add 1 extra for preceeding space   */
-    
+    					/* add 1 extra for preceding space   */
+
     make_newline = (wrap && (first_item || linep > max_line_len + 2));
     					/* NEVER new line in no-wrap mode */
-    
+
     if (make_newline) {			/* start new line, if needed */
         printf ("\n");
 	indent_out();			/* same exact indentation as pr_att */
@@ -164,8 +169,8 @@ print_any_val(
     const void *valp		/* pointer to the value */
 	    )
 {
-    if (varp->has_fillval && 
-	(*(varp->tinfo->val_equals))((const nctype_t *)varp->tinfo, 
+    if (varp->has_fillval &&
+	(*(varp->tinfo->val_equals))((const nctype_t *)varp->tinfo,
 				     (const void*)varp->fillvalp, valp) ) {
 	sbuf_cpy(sb, FILL_STRING);
     } else {
@@ -235,7 +240,7 @@ pr_any_att_vals(
     sbuf_free(sb);
 }
 
-/* 
+/*
  * Prints brief annotation for a row of data values
  */
 static void
@@ -243,7 +248,7 @@ annotate_brief(
     const ncvar_t *vp,		/* variable */
     const size_t *cor,		/* corner coordinates */
     size_t vdims[]		/* variable dimension sizes */
-    ) 
+    )
 {
     int vrank = vp->ndims;
     int id;
@@ -290,7 +295,7 @@ annotate(
 {
     int vrank = vp->ndims;
     int id;
-    
+
     /* print indices according to data_lang */
 /*     printf("  // %s(", vp->name); */
     printf("  // ");
@@ -409,7 +414,7 @@ upcorner(
     return ret;
 }
 
-/*  Print data values for variable varid.  
+/*  Print data values for variable varid.
  *
  * Recursive to handle possibility of variables with multiple
  * unlimited dimensions, for which the CDL syntax requires use of "{"
@@ -417,7 +422,7 @@ upcorner(
  * in a simple linear list of values.
  */
 static int
-print_rows(     
+print_rows(
     int level,          /* 0 at top-level, incremented for each recursive level */
     int ncid,		/* netcdf id */
     int varid,		/* variable id */
@@ -427,7 +432,7 @@ print_rows(
     size_t edg[],      	/* edges of hypercube */
     void *vals,   	/* allocated buffer for ncols values in a row */
     int marks_pending	/* number of pending closing "}" record markers */
-    ) 
+    )
 {
     int rank = vp->ndims;
     size_t ncols = rank > 0 ? vdims[rank - 1] : 1; /* number of values in a row */
@@ -442,7 +447,7 @@ print_rows(
 	inc *= vdims[i];
     }
     if(mark_record) { /* the whole point of this recursion is printing these "{}" */
-	lput("{");
+	lput(LBRACE);
 	marks_pending++;	/* matching "}"s to emit after last "row" */
     }
     if(rank - level > 1) {     	/* this level is just d0 next levels */
@@ -455,11 +460,11 @@ print_rows(
 	local_cor[level] = 0;
 	local_edg[level] = 1;
 	for(i = 0; i < d0 - 1; i++) {
-	    print_rows(level + 1, ncid, varid, vp, vdims, 
+	    print_rows(level + 1, ncid, varid, vp, vdims,
 		       local_cor, local_edg, vals, 0);
 	    local_cor[level] += 1;
 	}
-	print_rows(level + 1, ncid, varid, vp, vdims, 
+	print_rows(level + 1, ncid, varid, vp, vdims,
 		   local_cor, local_edg, vals, marks_pending);
 	free(local_edg);
 	free(local_cor);
@@ -473,7 +478,7 @@ print_rows(
 	NC_CHECK(nc_get_vara(ncid, varid, cor, edg, (void *)valp));
 
 	/* Test if we should treat array of chars as strings along last dimension  */
-	if(vp->type == NC_CHAR && (vp->fmt == 0 || STREQ(vp->fmt,"%s") || STREQ(vp->fmt,""))) {
+	if(vp->type == NC_CHAR && (vp->fmt == 0 || NCSTREQ(vp->fmt,"%s") || NCSTREQ(vp->fmt,""))) {
 	    pr_tvals(vp, ncols, vals, cor);
 	} else {			/* for non-text variables */
 	    for(i=0; i < d0 - 1; i++) {
@@ -489,28 +494,31 @@ print_rows(
 	    }
 	    print_any_val(sb, vp, (void *)valp);
 	}
+        /* In case vals has memory hanging off e.g. vlen or string, make sure to reclaim it */
+        NC_CHECK(nc_reclaim_data(ncid,vp->type,vals,ncols));
+
 	/* determine if this is the last row */
 	lastrow = true;
 	for(j = 0; j < rank - 1; j++) {
-	    if (cor[j] != vdims[j] - 1) {
+            if (cor[j] != vdims[j] - 1) {
 		lastrow = false;
 		break;
-	    }
+            }
 	}
 	if (formatting_specs.full_data_cmnts) {
-	    for (j = 0; j < marks_pending; j++) {
-		sbuf_cat(sb, "}");
-	    }
-	    printf("%s", sbuf_str(sb));
-	    lastdelim (0, lastrow);
-	    annotate (vp, cor, 0L);
+      for (j = 0; j < marks_pending; j++) {
+		sbuf_cat(sb, RBRACE);
+      }
+      printf("%s", sbuf_str(sb));
+      lastdelim (0, lastrow);
+      annotate (vp, cor, (d0 > 0 ? d0-1 : d0));
 	} else {
-	    for (j = 0; j < marks_pending; j++) {
-		sbuf_cat(sb, "}");
-	    }
-	    lput(sbuf_str(sb));
-	    lastdelim2 (0, lastrow);
-	}    
+            for (j = 0; j < marks_pending; j++) {
+		sbuf_cat(sb, RBRACE);
+            }
+      	    lput(sbuf_str(sb));
+            lastdelim2 (0, lastrow);
+	}
     }
     sbuf_free(sb);
     return NC_NOERR;
@@ -533,7 +541,6 @@ vardata(
     int id;
     size_t nels;
     size_t ncols;
-    size_t nrows;
     int vrank = vp->ndims;
 
     int level = 0;
@@ -576,9 +583,8 @@ vardata(
 	if (vrank > 1)
 	  add[vrank-2] = 1;
     }
-    nrows = nels/ncols;		/* number of "rows" */
     vals = emalloc(ncols * vp->tinfo->size);
-    
+
     NC_CHECK(print_rows(level, ncid, varid, vp, vdims, cor, edg, vals, marks_pending));
     free(vals);
     free(cor);
@@ -753,7 +759,7 @@ vardatax(
     }
     nrows = nels/ncols;		/* number of "rows" */
     vals = emalloc(ncols * vp->tinfo->size);
-    
+
     for (ir = 0; ir < nrows; ir++) {
 	size_t corsav = 0;
 	bool_t lastrow;
@@ -767,16 +773,16 @@ vardatax(
 	    edg[vrank-1] = ncols;
 	NC_CHECK(nc_get_vara(ncid, varid, cor, edg, vals) );
 	/* Test if we should treat array of chars as a string  */
-	if(vp->type == NC_CHAR && 
-	   (vp->fmt == 0 || STREQ(vp->fmt,"%s") || STREQ(vp->fmt,""))) {
+	if(vp->type == NC_CHAR &&
+	   (vp->fmt == 0 || NCSTREQ(vp->fmt,"%s") || NCSTREQ(vp->fmt,""))) {
 	    pr_tvalsx(vp, ncols, 0, lastrow, (char *) vals);
 	} else {
 	    pr_any_valsx(vp, ncols, 0, lastrow, vals);
 	}
-	
+
 	if (vrank > 0)
 	    cor[vrank-1] += ncols;
-	
+
 	if (vrank > 0)
 	  cor[vrank-1] = corsav;
 	if (ir < nrows-1)
